@@ -26,6 +26,8 @@ if sys.version_info < (3, 0):
     _binary_type = str
 else:
     _binary_type = bytes
+    # pylint: disable=redefined-builtin
+    long = int
 
 def _to_bytes_2and3(s):
     return s if isinstance(s, _binary_type) else s.encode('utf-8')
@@ -177,6 +179,30 @@ def _raise_for_status(r):
         raise exceptions.DXFUnauthorizedError()
     r.raise_for_status()
 
+class _ReportingFile(object):
+    def __init__(self, dgst, f, cb):
+        self._dgst = dgst
+        self._f = f
+        self._cb = cb
+        self._size = requests.utils.super_len(f)
+    # define __iter__ so requests thinks we're a stream
+    # (models.py, PreparedRequest.prepare_body)
+    def __iter__(self):
+        assert not "called"
+    # define fileno, tell and mode so requests can find length
+    # (utils.py, super_len)
+    def fileno(self):
+        return self._f.fileno()
+    def tell(self):
+        return self._f.tell()
+    @property
+    def mode(self):
+        return self._f.mode
+    def read(self, n):
+        chunk = self._f.read(n)
+        self._cb(self._dgst, chunk, self._size)
+        return chunk
+
 class DXFBase(object):
     # pylint: disable=too-many-instance-attributes
     """
@@ -315,7 +341,7 @@ class DXF(DXFBase):
     def _request(self, method, path, **kwargs):
         return super(DXF, self)._base_request(method, self._repo + '/' + path, **kwargs)
 
-    def push_blob(self, filename):
+    def push_blob(self, filename, progress=None):
         """
         Upload a file to the registry and return its (SHA-256) hash.
 
@@ -329,6 +355,13 @@ class DXF(DXFBase):
         :returns: Hash of file's content.
         """
         dgst = hash_file(filename)
+        try:
+            self._request('head', 'blobs/sha256:' + dgst)
+            return dgst
+        except requests.exceptions.HTTPError as ex:
+            # pylint: disable=no-member
+            if ex.response.status_code != requests.codes.not_found:
+                raise
         r = self._request('post', 'blobs/uploads/')
         upload_url = r.headers['Location']
         url_parts = list(urlparse.urlparse(upload_url))
@@ -338,27 +371,50 @@ class DXF(DXFBase):
         url_parts[0] = 'http' if self._insecure else 'https'
         upload_url = urlparse.urlunparse(url_parts)
         with open(filename, 'rb') as f:
-            r = self._base_request('put', upload_url, data=f)
+            self._base_request('put',
+                               upload_url,
+                               data=_ReportingFile(dgst, f, progress) if progress else f)
         return dgst
 
-    def pull_blob(self, digest):
+    # pylint: disable=no-self-use
+    def pull_blob(self, digest, size=False):
         """
         Download a blob from the registry given the hash of its content.
 
         :param digest: Hash of the blob's content.
         :type digest: str
 
+        :param size: Whether to return the size of the blob too
+        :type size: bool
+
         :rtype: iterator
-        :returns: Byte string iterator over the file's content.
+        :returns: If ```size``` is falsey, a byte string iterator over the file's content. If ```size``` is truthy, a tuple containing the iterator and the blob's size.
         """
         r = self._request('get', 'blobs/sha256:' + digest, stream=True)
-        sha256 = hashlib.sha256()
-        for chunk in r.iter_content(8192):
-            sha256.update(chunk)
-            yield chunk
-        dgst = sha256.hexdigest()
-        if dgst != digest:
-            raise exceptions.DXFDigestMismatchError(dgst, digest)
+        # pylint: disable=too-few-public-methods
+        class Chunks(object):
+            def __iter__(self):
+                sha256 = hashlib.sha256()
+                for chunk in r.iter_content(8192):
+                    sha256.update(chunk)
+                    yield chunk
+                dgst = sha256.hexdigest()
+                if dgst != digest:
+                    raise exceptions.DXFDigestMismatchError(dgst, digest)
+        return (Chunks(), long(r.headers['content-length'])) if size else Chunks()
+
+    def blob_size(self, digest):
+        """
+        Return the size of a blob in the registry given the hash of its content.
+
+        :param digest: Hash of the blob's content.
+        :type digest: str
+
+        :rtype: long
+        :returns: Whether the blob exists.
+        """
+        r = self._request('head', 'blobs/sha256:' + digest)
+        return long(r.headers['content-length'])
 
     def del_blob(self, digest):
         """
@@ -437,7 +493,8 @@ class DXF(DXFBase):
     def get_alias(self,
                   alias=None,
                   manifest=None,
-                  verify=True):
+                  verify=True,
+                  sizes=False):
         """
         Get the blob hashes assigned to an alias.
 
@@ -450,8 +507,11 @@ class DXF(DXFBase):
         :param verify: Whether to verify the integrity of the alias definition in the registry itself. You almost definitely won't need to change this from the default (``True``).
         :type verify: bool
 
+        :param sizes: Whether to return sizes of the blobs along with their hashes
+        :type sizes: bool
+
         :rtype: list
-        :returns: A list of blob hashes (strings) which are assigned to the alias.
+        :returns: If ```sizes``` is falsey, a list of blob hashes (strings) which are assigned to the alias. If ```sizes``` is truthy, a list of (hash,size) tuples for each blob.
         """
         if alias:
             r = self._request('get', 'manifests/' + alias)
@@ -459,7 +519,13 @@ class DXF(DXFBase):
             dcd = r.headers['docker-content-digest']
         else:
             dcd = None
-        return _verify_manifest(manifest, dcd, verify)
+        dgsts = _verify_manifest(manifest, dcd, verify)
+        if not sizes:
+            return dgsts
+        # V2 Schema 2 will put the size in the manifest, so we wouldn't need
+        # to make separate requests to get the size of each blob.
+        # Instead, we could get _verify_manifest to return them.
+        return [(dgst, self.blob_size(dgst)) for dgst in dgsts]
 
     def del_alias(self, alias):
         """
