@@ -2,12 +2,14 @@
 import os
 import argparse
 import sys
+import tqdm
 import dxf
 import dxf.exceptions
 
 _choices = ['auth',
             'push-blob',
             'pull-blob',
+            'blob-size',
             'del-blob',
             'set-alias',
             'get-alias',
@@ -41,11 +43,11 @@ def doit(args, environ):
         dxf_obj = dxf.DXF(environ['DXF_HOST'],
                           args.repo,
                           _auth,
-                          environ.get('DXF_INSECURE'))
+                          environ.get('DXF_INSECURE') == '1')
     else:
         dxf_obj = dxf.DXFBase(environ['DXF_HOST'],
                               _auth,
-                              environ.get('DXF_INSECURE'))
+                              environ.get('DXF_INSECURE') == '1')
 
     def _doit():
         # pylint: disable=too-many-branches
@@ -68,7 +70,18 @@ def doit(args, environ):
                 _parser.error('too many arguments')
             if len(args.args) == 2 and not args.args[1].startswith('@'):
                 _parser.error('invalid alias')
-            dgst = dxf_obj.push_blob(args.args[0])
+            if environ.get('DXF_PROGRESS') == '1':
+                bars = {}
+                def progress(dgst, chunk, size):
+                    if len(chunk) > 0:
+                        if dgst not in bars:
+                            bars[dgst] = tqdm.tqdm(desc=dgst[0:8], total=size, leave=True)
+                        bars[dgst].update(len(chunk))
+                    elif dgst in bars:
+                        bars[dgst].close()
+            else:
+                progress = None
+            dgst = dxf_obj.push_blob(args.args[0], progress)
             if len(args.args) == 2:
                 dxf_obj.set_alias(args.args[1][1:], dgst)
             print(dgst)
@@ -81,8 +94,30 @@ def doit(args, environ):
                                   if name.startswith('@') else [name]
                                   for name in args.args])
             for dgst in dgsts:
-                for chunk in dxf_obj.pull_blob(dgst):
+                it, size = dxf_obj.pull_blob(dgst, size=True)
+                # pylint: disable=blacklisted-name
+                if environ.get('DXF_PROGRESS') == '1':
+                    bar = tqdm.tqdm(desc=dgst[0:8], total=size, leave=True)
+                else:
+                    bar = None
+                for chunk in it:
+                    if bar is not None:
+                        bar.update(len(chunk))
                     sys.stdout.write(chunk)
+                if bar is not None:
+                    bar.close()
+
+        elif args.op == 'blob-size':
+            if len(args.args) == 0:
+                sizes = [dxf_obj.get_alias(manifest=sys.stdin.read(),
+                                           sizes=True)]
+            else:
+                sizes = [dxf_obj.get_alias(name[1:], sizes=True)
+                         if name.startswith('@') else
+                         [(name, dxf_obj.blob_size(name))]
+                         for name in args.args]
+            for tuples in sizes:
+                print(sum([size for _, size in tuples]))
 
         elif args.op == 'del-blob':
             if len(args.args) == 0:
